/*globals define*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/shopmailtype/ui/editmailaddress',
    'ep/ajax',
    'testsuite/qunit-fixture!de_epages/shopmailtype/ui/editmailaddress-test',

    '$ready!'
], function (QUnit, $, de_epages, ep, fixture) {
    'use strict';

    QUnit.module("de_epages.shopmailtype.ui.editmailaddress", {
        setup: function () {
            fixture.init();
        },

        teardown: function () {
            fixture.destroy();
        }
    });

    QUnit.test("de_epages.shopmailtype.ui.editmailaddress", function () {

        // Check prototype methods.
        $.each(['_create', '_setOption', 'destroy', '_deleteMail', '_addMail', '_apply', '_close'], function (i, fn) {

            QUnit.equal(
                $.isFunction($.ui.shopmailtypeUiEditmailaddress.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        // Create.
        var $from = $('#ep-edit-FROM').shopmailtypeUiEditmailaddress(),
            $cc = $('#ep-edit-CC').shopmailtypeUiEditmailaddress(),
            fromForm,
            fromApplyButton,
            fromSetDefault,
            ccForm,
            ccApplyButton,
            ccSetDefault,
            ccAddMail;

        QUnit.equal(
            typeof $from.data('uiUiSimpledialog') === 'object',
            true,
            "typeof $from.data('uiUiSimpledialog') === 'object'"
        );

        QUnit.equal(
            typeof $cc.data('uiUiSimpledialog') === 'object',
            true,
            "typeof $cc.data('uiUiSimpledialog') === 'object'"
        );

        // Open *$from* dialog.
        $from.trigger('click');

        fromForm = $($from).data('uiUiSimpledialog').content.find('form');
        fromApplyButton = $($from).data('uiUiSimpledialog').buttons.find('button');
        fromSetDefault = $($from).data('uiUiSimpledialog').content.find('input[type="checkbox"]');

        QUnit.equal(
            fromForm.serialize(),
            'email=ttester%40epages.com&name=TTester',
            'fromForm.serialize()'
        );

        // Change value of mail address to empty string.
        fromForm.find('input[name="email"]').val('');

        QUnit.equal(
            fromForm.serialize(),
            'email=&name=TTester',
            'fromForm.serialize()'
        );

        // Try applying changes.
        fromApplyButton.trigger('click');

        QUnit.equal(
            $('input[name="FROM"]').val(),
            '"TTester" <ttester@epages.com>',
            "$('input[name=\"FROM\"]').val()"
        );

        // Change value of mail address to none empty string.
        fromForm.find('input[name="email"]').val('tonytester@epages.com');

        QUnit.equal(
            fromForm.serialize(),
            'email=tonytester%40epages.com&name=TTester',
            'fromForm.serialize()'
        );

        // Try applying changes.
        fromApplyButton.trigger('click');

        QUnit.equal(
            $('input[name="FROM"]').val(),
            'TTester <tonytester@epages.com>',
            "$('input[name=\"FROM\"]').val()"
        );

        // Check the *setDefault* checkbox.
        fromSetDefault.prop('checked', true);

        // Fake ajax.
        ep.ajax = function (o) {
            var result = $.Deferred();

            // Check the data.
            QUnit.deepEqual({
                DefaultEmail: o.data.DefaultEmail,
                DefaultName: o.data.DefaultName
            }, {
                DefaultEmail: 'tonytester@epages.com',
                DefaultName: 'TTester'
            },
                'Submitted *data*:'
                );

            result.resolve();

            return result;
        };

        // Try applying changes.
        fromApplyButton.trigger('click');

        // Open *$from* dialog.
        $cc.trigger('click');

        ccForm =  $($cc).data('uiUiSimpledialog').content.find('form');
        ccApplyButton = $($cc).data('uiUiSimpledialog').buttons.find('button');
        ccSetDefault = $($cc).data('uiUiSimpledialog').content.find('input[type="checkbox"]');
        ccAddMail = $($cc).data('uiUiSimpledialog').content.find('footer');

        QUnit.equal(
            ccForm.serialize(),
            'email=&name=',
            'ccForm.serialize()'
        );

        QUnit.equal(
            ccSetDefault.length === 0,
            true,
            '*ccForm* has no *setDefault* checkbox'
        );

        // Try adding a second input field.
        ccAddMail.children().trigger('click');

        QUnit.equal(
            ccForm.serialize(),
            'email=&name=&email=&name=',
            'ccForm.serialize()'
        );

        // Delete an input set.
        ccForm.find('[title="Delete"]').first().trigger('click');

        QUnit.equal(
            ccForm.serialize(),
            'email=&name=',
            'ccForm.serialize()'
        );

        // Try applying changes.
        fromApplyButton.trigger('click');

        QUnit.equal(
            $('input[name="CC"]').val(),
            '',
            "$('input[name=\"CC\"]').val()"
        );

        // Destroy.
        $($from).shopmailtypeUiEditmailaddress('destroy');
        $($cc).shopmailtypeUiEditmailaddress('destroy');

        QUnit.equal(
            $._data($from[0], 'events'),
            undefined,
            '*click* handler removed'
        );

        QUnit.equal(
            $from.data('uiUiSimpledialog'),
            undefined,
            '*uiSimpledialog* data attribute removed'
        );

        QUnit.deepEqual(
            $from.data('de_epagesShopmailtypeUiEditmailaddress'),
            $.parseJSON($from.attr('data-de_epages-shopmailtype-ui-editmailaddress') || "null"),
            '*shopmailtypeUiEditmailaddress* data attribute removed'
        );

        QUnit.equal(
            $._data($cc[0], 'events'),
            undefined,
            '*click* handler removed'
        );

        QUnit.equal(
            $cc.data('uiUiSimpledialog'),
            undefined,
            '*uiSimpledialog* data attribute removed'
        );

        QUnit.deepEqual(
            $cc.data('de_epagesShopmailtypeUiEditmailaddress'),
            $.parseJSON($cc.attr('data-de_epages-shopmailtype-ui-editmailaddress') || "null"),
            '*shopmailtypeUiEditmailaddress* data attribute removed'
        );
    });
});