/*globals define, window*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/tooltip-test',
    'jquery',
    'ep/ui/tooltip',

    '$ready!'
], function (QUnit, fixture, $) {

    'use strict';

    QUnit.module("ep/ui/tooltip", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.test('Check tooltips methods', 9, function () {
        $.each(['_create', '_init', '_enter', '_leave', '_show', 'show', '_hide', 'hide', 'destroy'], function () {
            QUnit.ok(
                $.isFunction($.ui.uiTooltip.prototype[this]),
                this + ' is a method of the tooltip widget'
            );
        });
    });

    QUnit.asyncTest('Check tooltips options', 6, function () {
        var tooltip = $('#tooltip').uiTooltip({
            addClass: 'my-test-class',
            closeButton: true,
            context: '#context',
            interactive: true
        });

        $('#context').trigger('mouseenter');

        window.setTimeout(function () {
            QUnit.strictEqual(
                tooltip.parent().css('display'),
                'block',
                'Show tooltip after mouseenter event on context node'
            );

            QUnit.strictEqual(
                tooltip.data('uiUiTooltip').options.context,
                '#context',
                'Context of the tooltip is set to the *#content* node'
            );

            QUnit.strictEqual(
                $('.my-test-class').length,
                1,
                'Class *my-test-class* is added to tooltip wrapper'
            );

            QUnit.strictEqual(
                $('.ep-uiTooltip-closeButton').length,
                1,
                "Close button is added to tooltip"
            );

            tooltip.trigger('click');

            window.setTimeout(function () {
                QUnit.strictEqual(
                    tooltip.parent().css('display'),
                    'block',
                    'Tooltip is still displayed after a click in it because it is set to interactive'
                );

                tooltip.uiTooltip('destroy');

                QUnit.strictEqual(
                    tooltip.data('uiUiTooltip'),
                    undefined,
                    'Tooltip is successfully destroyed'
                );

                QUnit.start();

            }, 400);
        }, 400);
    });

    QUnit.asyncTest('Check tooltips event handling', 3, function () {
        var tooltip = $('#tooltip').uiTooltip({
            context: '#context'
        });

        $('#context').trigger('mouseenter');

        window.setTimeout(function () {
            QUnit.strictEqual(
                tooltip.parent().css('display'),
                'block',
                'Show tooltip after mouseenter event on context node'
            );

            QUnit.strictEqual(
                tooltip.parent().hasClass('ep-uiTooltip'),
                true,
                'Add wrapper with class ep-uiTooltip'
            );

            $('#context').trigger('mouseleave');

            window.setTimeout(function () {
                QUnit.strictEqual(
                    tooltip.parent().css('display'),
                    'none',
                    'Hide tooltip after mouseleave event on context node'
                );

                QUnit.start();

            }, 1000);

        }, 400);
    });
});