/*global define, window*/
/*jslint nomen:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!./googleplus-test.html',
    'jquery',

    './googleplus',
    '$ready!'
], function (QUnit, fixture, $) {
    'use strict';

    QUnit.module('de_epages/externalcontent/ui/googleplus', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.test('Test googleplus methods', 2, function () {
        $.each(['_create', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiGoogleplus.prototype[fn]),
                true,
                fn + ' is a method of googleplus'
            );
        });
    });

    QUnit.test('Test googleplus', 18, function () {
        window.container = null;
        window.params = null;

        //Define Google API for the Test
        window.gapi = {
            plusone: {
                render: function (_container, _params) {
                    window.container = _container;
                    window.params = _params;
                }
            }
        };

        $.ajax = function (o) {
            return {
                done: function (_func) {
                    _func();
                }
            };
        };

        //Create 1st. Google+ Button
        $("#googlebutton1").externalcontentUiGoogleplus({
            href: "www.testurl1.de",
            layout: "standard",
            type: "bubble"
        });

        QUnit.equal(
            window.params.href,
            "www.testurl1.de",
            "the first Button got the right parameter href = <www.testurl1.de>"
        );

        QUnit.equal(
            window.params.size,
            "standard",
            "the first Button got the right parameter size = <standard>"
        );

        QUnit.equal(
            window.params.annotation,
            "bubble",
            "the first Button got the right parameter annotation = <bubble>"
        );

        QUnit.equal(
            $(window.container).parent().attr("id"),
            $("#googlebutton1").attr("id"),
            "the button is appended to the correct div"
        );

        //Create 2nd. Google+ Button
        $("#googlebutton2").externalcontentUiGoogleplus({
            href: "www.testurl2.de",
            layout: "medium",
            type: "inline"
        });

        QUnit.equal(
            window.params.href,
            "www.testurl2.de",
            "the first Button got the right parameter href = <www.testurl2.de>"
        );

        QUnit.equal(
            window.params.size,
            "medium",
            "the first Button got the right parameter size = <medium>"
        );

        QUnit.equal(
            window.params.annotation,
            "inline",
            "the first Button got the right parameter annotation = <inline>"
        );

        QUnit.equal(
            $(window.container).parent().attr("id"),
            $("#googlebutton2").attr("id"),
            "the button is appended to the correct div"
        );

        //Create 3rd. Google+ Button
        $("#googlebutton3").externalcontentUiGoogleplus({
            href: "www.testurl3.de",
            layout: "small",
            type: "none"
        });

        QUnit.equal(
            window.params.href,
            "www.testurl3.de",
            "the first Button got the right parameter href = <www.testurl3.de>"
        );

        QUnit.equal(
            window.params.size,
            "small",
            "the first Button got the right parameter size = <small>"
        );

        QUnit.equal(
            window.params.annotation,
            "none",
            "the first Button got the right parameter annotation = <none>"
        );

        QUnit.equal(
            $(window.container).parent().attr("id"),
            $("#googlebutton3").attr("id"),
            "the button is appended to the correct div"
        );

        //Gets all the googlebutton and destroys each of them with the Widget method .externalcontentUiGoogleplus('destroy')
        $("div[id^=googlebutton]").each(function () {
            $(this).externalcontentUiGoogleplus("destroy");
        });

        //Tests if the Google Classes have been deleeted successfully
        $("div[id^=googlebutton]").each(function () {
            QUnit.equal(
                ($(this).attr("class") === ""),
                true,
                "the destroy method has successfully deleted the class SocialWeb from the <<Div>>  with ID " + $(this).attr("id")
            );
        });

        //Tests if the appended Containers have been deleeted successfully
        var ChildNodes = 1;
        $("div[id^=googlebutton]").each(function () {
            QUnit.equal(
                $(this).children().length,
                0,
                "the destroy method has successfully disappended all child elements, length of child elements from the <<Div>> with ID " + $(this).attr("id") + " = 0"
            );
        });
    });
});